@@ -4,7 +4,7 @@ use warnings FATAL => 'all';
 use base 'Exporter';
 use Carp;
 
-our $VERSION = '0.002003';
+our $VERSION = '0.003000';
 $VERSION = eval $VERSION;
 
 our @EXPORT_OK = qw(
@@ -16,6 +16,8 @@ our @EXPORT_OK = qw(
     quote_system_cmd
     quote_literal
     cmd_escape
+    unquote_native
+    cmd_unescape
 );
 our %EXPORT_TAGS = (all => [@EXPORT_OK]);
 
@@ -94,10 +96,62 @@ sub _has_shell_metachars {
 
     return 1
         if $string =~ /%/;
-    $string =~ s/(['"]).*?(\1|\z)//;
+    $string =~ s/(['"]).*?(\1|\z)//sg;
     return $string =~ /[<>|]/;
 }
 
+sub unquote_native {
+    local ($_) = @_;
+    my @argv;
+
+    my $length = length
+        or return @argv;
+
+    m/\G\s*/gc;
+
+    ARGS: until ( pos == $length ) {
+        my $quote_mode;
+        my $arg = '';
+        CHARS: until ( pos == $length ) {
+            if ( m/\G((?:\\\\)+)(?=\\?(")?)/gc ) {
+                if (defined $2) {
+                    $arg .= '\\' x (length($1) / 2);
+                }
+                else {
+                    $arg .= $1;
+                }
+            }
+            elsif ( m/\G\\"/gc ) {
+                $arg .= '"';
+            }
+            elsif ( m/\G"/gc ) {
+                if ( $quote_mode && m/\G"/gc ) {
+                    $arg .= '"';
+                }
+                $quote_mode = !$quote_mode;
+            }
+            elsif ( !$quote_mode && m/\G\s+\z?/gc ) {
+                last;
+            }
+            elsif ( m/\G(.)/sgc ) {
+                $arg .= $1;
+            }
+        }
+        push @argv, $arg;
+    }
+
+    return @argv;
+}
+
+sub cmd_unescape {
+    my ($string) = @_;
+
+    no warnings 'uninitialized';
+    $string =~ s/\^(.?)|([^^"]+)|("[^"]*(?:"|\z))/$1$2$3/gs;
+
+    return $string;
+}
+
 1;
 
 __END__
@@ -114,7 +168,7 @@ Win32::ShellQuote - Quote argument lists for Win32
 
 =head1 DESCRIPTION
 
-Quotes argument lists to be used in Win32 in a several different
+Quotes argument lists to be used in Win32 in several different
 situations.
 
 Windows passes its arguments as a single string instead of an array
